use strict;
use warnings;

use CodeCounter;

my $total;

foreach my $path (@INC){
    my $count;
    $count = base_count($path, 'conf/standards.xml');
    print "LIB: $path COUNT: $count\n";        
    $total += $count;
}

print "TOTAL COUNT: $total\n";